/* Example input for ch2c+h, a utility to extract .C and .H files from a
 * common .CH file. 
 * Author: Jaap-Henk Hoepman (jhh@xs4all.nl)
 *
 * $Id: ch2c+h.ch,v 1.1 2000/01/13 08:04:29 hoepman Exp $
 *
 * Running the following command
 *
 * ch2c+h cfile="c.c" hfile="h.h" ch2c+h.ch
 *
 * will produce the C source file c.c and the C header file h.h
 */

@C
/* include some header files in the .C file */
#include <stdlib.h>

@H
/* now define the necessary types; these go in the .H file */
typedef char ELT ;

typedef struct
{ int len ;
  ELT *store ;
} ARRAY ;

/* An external; variable needs to be `visible' in both */
@EXTERN
ARRAY bigarray ;

/* Header files contain prototypes of all externally visible functions
 * implemented in the .C file. The @PROTO directive saves you typing and 
 * avoids mismatches when the .C file is changed and the .H is not updated
 * to reflect this change
 */
@PROTO
ELT get_element (ARRAY *a, int i)
{
  return ( a->store[i] ) ;
}

/* For inline functions, the code needs to be available in both the
 * .C file and the .H file.
 */
@H
extern inline    /* make sure code is generated for only one copy */
@CH
void set_element ( ARRAY *a, int i, ELT e)
{
  assert(i > 0) ;
  assert(i < a->len) ;
  a->store[i] = e ;
}



